/*
 *  libMirage: NULL fragment: Fragment object
 *  Copyright (C) 2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "fragment-null.h"


/******************************************************************************\
 *                                 Misc stuff                                 *
\******************************************************************************/
/* Nice macro that gives for loop on GList */
#define G_LIST_FOR_EACH(cursor,list) \
    for ((cursor) = (list); (cursor); (cursor) = (cursor)->next)

#define SANITY_CHECK(function)                                          \
    if (!self) {                                                        \
        mirage_error(MIRAGE_E_OBJISNULL, error);                        \
        return FALSE;                                                   \
    }                                                                   \
    if (!MIRAGE_IS_FRAGMENT(self)) {                                    \
        mirage_error(MIRAGE_E_INVALIDOBJ, error);                       \
        return FALSE;                                                   \
    }                                                                   \
    if (MIRAGE_FRAGMENT_GET_PRIVATE(self)->dispose_has_run) {           \
        mirage_error(MIRAGE_E_OBJDISPOSED, error);                      \
        return FALSE;                                                   \
    }                                                                   \
    if (!MIRAGE_FRAGMENT_GET_CLASS(self)->function) {                   \
        mirage_error(MIRAGE_E_NOFUNCTION, error);                       \
        return FALSE;                                                   \
    }

#define CHECK_ARG(param)                                                \
    if (!param) {                                                       \
        mirage_error(MIRAGE_E_INVALIDARG, error);                       \
        return FALSE;                                                   \
    }


/******************************************************************************\
 *                    Implementations of virtual functions                    *
\******************************************************************************/
static gboolean __mirage_fragment_null_get_fragment_type (MIRAGE_Fragment *self, gchar **fragment_type, GError **error) {
    CHECK_ARG(fragment_type);
    /* Return type string */
    *fragment_type = g_strdup(mirage_plugin_info.fragment_id);
    return TRUE;
}

static gboolean __mirage_fragment_null_use_the_rest_of_file (MIRAGE_Fragment *self, GError **error) {
    /* No file, nothing to use */
    return TRUE;
}

static gboolean __mirage_fragment_null_read_main_data (MIRAGE_Fragment *self, gint address, guint8 *buf, gint *length, GError **error) {
    /* Nothing to read */
    MIRAGE_DEBUG(self, MIRAGE_DEBUG_FRAGMENT, "%s: no data in NULL fragment\n", __func__);
    if (length) {
        *length = 0;
    }    
    return TRUE;
}

static gboolean __mirage_fragment_null_read_subchannel_data (MIRAGE_Fragment *self, gint address, guint8 *buf, gint *length, GError **error) {
    /* Nothing to read */
    MIRAGE_DEBUG(self, MIRAGE_DEBUG_FRAGMENT, "%s: no data in NULL fragment\n", __func__);
    if (length) {
        *length = 0;
    }
    return TRUE;
}


/* Object init */
static void mirage_fragment_null_class_init (gpointer g_class, gpointer g_class_data) {
    MIRAGE_FragmentClass *klass = MIRAGE_FRAGMENT_CLASS(g_class);
    
    /* Initialize MIRAGE_Fragment methods */
    klass->get_fragment_type = __mirage_fragment_null_get_fragment_type;
    klass->use_the_rest_of_file = __mirage_fragment_null_use_the_rest_of_file;
    klass->read_main_data = __mirage_fragment_null_read_main_data;
    klass->read_subchannel_data = __mirage_fragment_null_read_subchannel_data;
    
    return;
}

GType mirage_fragment_null_get_type (void) {
    static GType type = 0;
    if (type == 0) {
        static const GTypeInfo info = {
            sizeof(MIRAGE_Fragment_NULLClass),
            NULL,   /* base_init */
            NULL,   /* base_finalize */
            mirage_fragment_null_class_init,   /* class_init */
            NULL,   /* class_finalize */
            NULL,   /* class_data */
            sizeof(MIRAGE_Fragment_NULL),
            0,      /* n_preallocs */
            NULL    /* instance_init */
        };
        
        static const GInterfaceInfo interface_info = {
            NULL,   /* interface_init */
            NULL,   /* interface_finalize */
            NULL    /* interface_data */
        };
                
        type = g_type_register_static(MIRAGE_TYPE_FRAGMENT, "MIRAGE_Fragment_NULL", &info, 0);
        
        g_type_add_interface_static(type, MIRAGE_TYPE_FINTERFACE_NULL, &interface_info);
    }
    
    return type;
}
