/*
 *  libMirage: Constants
 *  Copyright (C) 2006-2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __LIBMIRAGE_DEFS_H__
#define __LIBMIRAGE_DEFS_H__

#define MIRAGE_ISRC_SIZE    12
#define MIRAGE_MCN_SIZE     13

/* Some flags for additional track data provided */
typedef enum {
    MIRAGE_TRACKF_CHAN = 0x01, /* 0 = 2 channel, 1 = four channel */
    MIRAGE_TRACKF_COPY = 0x02, /* 0 = no copy, 1 = copy */
    MIRAGE_TRACKF_EMPH = 0x04, /* 0 = no pre-emphasis, 1 = pre-emphasis */
} MIRAGE_TrackFlags;

/* Medium types */
typedef enum {
    MIRAGE_MEDIUM_CD  = 0x01, /* CD-ROM */
    MIRAGE_MEDIUM_DVD = 0x02, /* DVD-ROM */
    MIRAGE_MEDIUM_BD  = 0x03, /* BD Disc */
} MIRAGE_MediumType;

/* Session types... applies to CD-ROMs only I think */
typedef enum {
    MIRAGE_SESSION_CD_DA     = 0x00, /* CD AUDIO */
    MIRAGE_SESSION_CD_ROM    = 0x00, /* CD-ROM */
    MIRAGE_SESSION_CD_I      = 0x10, /* CD-I */
    MIRAGE_SESSION_CD_ROM_XA = 0x20, /* CD-ROM XA */
} MIRAGE_SessionType;

/* Track modes */
typedef enum {
    MIRAGE_MODE_MODE0       = 0x00, /* Mode 0 */
    MIRAGE_MODE_AUDIO       = 0x01, /* Audio */
    MIRAGE_MODE_MODE1       = 0x02, /* Mode 1 */
    MIRAGE_MODE_MODE2       = 0x03, /* Mode 2 Formless */
    MIRAGE_MODE_MODE2_FORM1 = 0x04, /* Mode 2 Form 1 */
    MIRAGE_MODE_MODE2_FORM2 = 0x05, /* Mode 2 Form 2 */
    MIRAGE_MODE_MODE2_MIXED = 0x06, /* Mode 2 Mixed */
} MIRAGE_TrackMode;

/* Lead-in and Lead-out track constants (used in get_track_by_number) */
typedef enum {
    MIRAGE_TRACK_LEADIN  = 0x00,
    MIRAGE_TRACK_LEADOUT = 0xAA,
} MIRAGE_TrackConstant;

/* Sector valid data flags */
typedef enum {
    MIRAGE_VALID_SYNC      = 0x01,
    MIRAGE_VALID_HEADER    = 0x02,
    MIRAGE_VALID_SUBHEADER = 0x04,
    MIRAGE_VALID_DATA      = 0x08,
    MIRAGE_VALID_EDC_ECC   = 0x10,
    MIRAGE_VALID_SUBCHAN   = 0x20,
} MIRAGE_SectorValidData;

#endif /* __LIBMIRAGE_DEFS_H__ */
