/*
 *  libMirage: Error handling
 *  Copyright (C) 2006-2007 Rok Mandeljc
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef __MIRAGE_ERROR_H__
#define __MIRAGE_ERROR_H__


G_BEGIN_DECLS

#define MIRAGE_ERROR mirage_error_quark()
GQuark mirage_error_quark (void);

/* Error codes */
enum {
    /* General */
    MIRAGE_E_OBJISNULL          = 0xDEAD000,
    MIRAGE_E_OBJDISPOSED        = 0xDEAD001,
    MIRAGE_E_INVALIDOBJ         = 0xDEAD002,
    MIRAGE_E_INVALIDARG         = 0xDEAD003,
    MIRAGE_E_NOTIMPL            = 0xDEAD004,
    MIRAGE_E_NOFUNCTION         = 0xDEAD005,
    
    MIRAGE_E_INVALIDOBJTYPE     = 0xDEAD006,

    MIRAGE_E_DATANOTSET         = 0xDEAD008,
    MIRAGE_E_DATAFIXED          = 0xDEAD009,

    MIRAGE_E_NOTINLAYOUT        = 0xDEAD00A,

    MIRAGE_E_INDEXOUTOFRANGE    = 0xDEAD00B,
    MIRAGE_E_SECTOROUTOFRANGE   = 0xDEAD00C,
    
    /* Mirage */
    MIRAGE_E_NOPARSERFOUND      = 0xDEAD020,
    MIRAGE_E_NOFRAGMENTFOUND    = 0xDEAD021,
    MIRAGE_E_IMAGEFILE          = 0xDEAD022,
    
    /* Plugins */
    MIRAGE_E_PARSER             = 0xDEAD030,
    MIRAGE_E_SINGLEFILE         = 0xDEAD031,
    
    /* Object */
    MIRAGE_E_NODEBUGCONTEXT     = 0xDEAD040,
    MIRAGE_E_NOPARENT           = 0xDEAD041,
    MIRAGE_E_NOMIRAGE           = 0xDEAD042,

    /* Disc */    
    MIRAGE_E_SESSIONNOTFOUND    = 0xDEAD050,
    MIRAGE_E_SESSIONEXISTS      = 0xDEAD051,
    MIRAGE_E_INVALIDMEDIUM      = 0xDEAD052,
    
    /* Session */
    MIRAGE_E_TRACKNOTFOUND      = 0xDEAD060,
    MIRAGE_E_LANGNOTFOUND       = 0xDEAD061,
    MIRAGE_E_TRACKEXISTS        = 0xDEAD062,
    MIRAGE_E_LANGEXISTS         = 0xDEAD063,
    
    /* Track */
    MIRAGE_E_INDEXNOTFOUND      = 0xDEAD070,
    MIRAGE_E_FRAGMENTNOTFOUND   = 0xDEAD071,
    
    /* Fragment */
    MIRAGE_E_FILENOTSET         = 0xDEAD080,
    MIRAGE_E_READFAILED         = 0xDEAD081,
    MIRAGE_E_DATAFILE           = 0xDEAD082,
    
    /* Sector */
    MIRAGE_E_SECTORTYPE         = 0xDEAD090,
    
    /* Language */
    MIRAGE_E_INVALIDPACKTYPE    = 0xDEAD0A0,
    MIRAGE_E_PACKNOTSET         = 0xDEAD0A1,
    
    
    MIRAGE_E_GENERIC            = 0xDEADFFF,
} mirage_error_codes;

/* Error function */
void mirage_error (gint errcode, GError **error);

G_END_DECLS

#endif /* __MIRAGE_ERROR_H__ */
